/*-
 * Copyright (c) 2004 Andrey Simonenko
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *  @(#)$Id: ipa_ctl.h,v 1.1 2005/12/03 10:27:59 simon Exp $
 */

#ifndef IPA_CTL_H
#define IPA_CTL_H

#define CTL_SOCKET_PERM_DEF	(S_IRUSR|S_IWUSR)	/* 0600, ctl_socket default value.  */
#define CTL_TIMEOUT_DEF		10			/* ctl_timeout default value. */

#include "ipactl.h"

#ifdef CTL_CHECK_CREDS

struct ctl_acl_elem {
	STAILQ_ENTRY(ctl_acl_elem) link; /* For list building. */
	char	*user;		/* User name or NULL. */
	char	*group;		/* Group name or NULL. */
	int	allowed;	/* Non-zero if allowed. */
};

struct ctl_acl_class {
	STAILQ_ENTRY(ctl_acl_class) link; /* For class list building. */
	STAILQ_HEAD(, ctl_acl_elem) list; /* Head of ctl elements list. */
	char	*class_name;	/* Class name. */
};

#define CTL_ACL_ELEM_NSIZE	10
#define CTL_ACL_ELEM_NALLOC	10

#define CTL_ACL_CLASS_NSIZE	10
#define CTL_ACL_CLASS_NALLOC	10

STAILQ_HEAD(ctl_acl_classes, ctl_acl_class);

extern ipa_mzone *ctl_acl_elem_mzone;
extern ipa_mzone *ctl_acl_class_mzone;
extern struct ctl_acl_classes ctl_acl_classes;
extern const struct ctl_acl_class *ctl_dump_acl;
extern const struct ctl_acl_class *ctl_freeze_acl;
extern const struct ctl_acl_class *ctl_stat_acl;
extern const struct ctl_acl_class *global_ctl_rule_acl;

#endif /* CTL_CHECK_CREDS */

extern ipa_mem_type *m_ctl;

extern int	ctl_listenfd;

extern int	ctl_enable;
extern char	*ctl_socket_path;
extern mode_t	ctl_socket_perm;
extern u_int	ctl_timeout;

extern char	*ctl_socket_path_default;

extern int	debug_ctl;

extern int	init_ctl(void);
extern int	deinit_ctl(void);

extern int	ctl_recv_query(void);
extern int	ctl_send_answer(void);

extern void	sig_alrm(int);

extern int	do_ctl_func_set_rule(struct rule *, const struct ctl_cmd_query *);

#ifdef WITH_LIMITS
extern int	do_ctl_func_expire(struct rule *, struct limit *, int);
extern int	do_ctl_func_restart(struct rule *, struct limit *, int);
extern int	do_ctl_func_set_limit(struct rule *, struct limit *, int, const struct ctl_cmd_query *);
#endif

#ifdef WITH_THRESHOLDS
extern int	do_ctl_func_set_threshold(struct rule *, struct threshold *, int, const struct ctl_cmd_query *);
#endif

#endif /* !IPA_CTL_H */
